from aws_xray_sdk.core import xray_recorder as xray


def xray_capture(func):
    """Wraps xray.capture(), allowing a class to opt in/out"""

    def wrapper(*args, **kwargs):
        if (
            len(args) > 0
            and isinstance(args[0], object)
            and callable(getattr(args[0], func.__name__, None))
            and not getattr(args[0], "_xray_capture", False)
        ):
            # "func" is most likely an object function on an object
            # that has not set its "_xray_capture" to True
            # Do not allow xray.capture()
            return func(*args, **kwargs)
        else:
            return xray.capture()(func)(*args, **kwargs)

    return wrapper



import argparse
import json
import re
from typing import Optional

from psi_utils.help_formatter import format_help
from psi_utils.json_utils import JsonEncoder


class ArgumentParser(argparse.ArgumentParser):
    def __init__(self, default_help_format: bool = False, **kwargs):
        super().__init__(**kwargs)
        self.__default_help_format = bool(default_help_format)

    def parse_args_and_prompt(self):
        args = self.parse_args()
        args_dict = {arg[0]: arg[1] for arg in vars(args).items()}

        user_input = validate_interactive_input(
            prompt=f"Running with args:\n{json.dumps(args_dict, indent=4, cls=JsonEncoder)}\nEnter y/n to continue:",
            allowed_values=["y", "n"],
        )
        return None if user_input != "y" else args

    def print_help(self, file=None):
        if self.__default_help_format:
            super().print_help(file=file)
        else:
            msg = format_help(self)
            if file:
                file.write(msg)
            else:
                print(msg)


def validate_interactive_input(
    prompt: str, regex: Optional[str] = None, allowed_values: list = []
) -> str:
    while True:
        user_input = input(f"{prompt}\n").lower().strip()
        if regex and re.match(regex, user_input):
            return user_input
        if allowed_values and user_input in allowed_values:
            return user_input

        msg = "\nInvalid input. "
        msg += f'Input must match regex "{regex}". ' if regex else ""
        msg += f"Input must be one of {allowed_values}." if allowed_values and not regex else ""
        msg += f"Or input must be one of {allowed_values}." if allowed_values and regex else ""
        print(msg)



import unittest.mock as mock


class _ANY_NOT_NONE(object):
    """A helper object similar to unittest.mock.ANY
    that compares equal to everything that isn't None.
    """

    def __eq__(self, other):
        return other is not None

    def __ne__(self, other):
        return other is None

    def __repr__(self):
        return "<ANY_NOT_NONE>"


ANY = mock.ANY
ANY_NOT_NONE = _ANY_NOT_NONE()


class MockFunction(mock.MagicMock):
    def __init__(self, fn_name):
        super().__init__(side_effect=self._side_effect)
        self._fn_name = fn_name
        self._expected_invocations = []
        self._reset_temp()

    def on(self, *args, **kwargs):
        self._temp_args = args
        self._temp_kwargs = kwargs
        return self

    def return_val(self, return_value):
        self._expected_invocations.append(
            {
                "args": self._temp_args,
                "kwargs": self._temp_kwargs,
                "return_value": return_value,
                "invoked": False,
            }
        )
        self._reset_temp()

    def raise_exception(self, exception):
        self._expected_invocations.append(
            {
                "args": self._temp_args,
                "kwargs": self._temp_kwargs,
                "exception": exception,
                "invoked": False,
            }
        )

    def assert_expectations(self):
        for expected in self._expected_invocations:
            args, kwargs = expected["args"], expected["kwargs"]
            assert expected[
                "invoked"
            ], f"Expected an invocation of function {self._fn_name} with args {args} and kwargs {kwargs}"

    def _side_effect(self, *args, **kwargs):
        for expected in self._expected_invocations:
            if self._args_match(expected["args"], args) and self._kwargs_match(
                expected["kwargs"], kwargs
            ):
                expected["invoked"] = True
                if "exception" in expected:
                    raise expected["exception"]
                else:
                    return expected["return_value"]
        raise Exception(
            f"Unexpected invocation of function '{self._fn_name}' with args: {list(args)} and kwargs: {kwargs}"
        )

    def _args_match(self, expected, actual):
        expected_lst, actual_lst = list(expected), list(actual)
        if len(expected) != len(actual):
            return False

        for index, expected_val in enumerate(expected_lst):
            if expected_val != actual_lst[index]:
                return False
        return True

    def _kwargs_match(self, expected, actual):
        for key in actual:
            if key not in expected:
                return False

        for key, expected_val in expected.items():
            if expected_val != actual.get(key, None):
                return False
        return True

    def _reset_temp(self):
        self._temp_args = ()
        self._temp_kwargs = {}


class UnexpectedFunctionInvocation(mock.MagicMock):
    def __init__(self, fn_name):
        super().__init__(side_effect=self._side_effect)
        self._fn_name = fn_name

    def _side_effect(self, *args, **kwargs):
        raise AttributeError(
            f"Unexpected call to function '{self._fn_name}' with args {list(args)} and kwargs {kwargs}"
        )


class MockApi:
    def on(self, fn_name, *args, **kwargs):
        if not self._hasattr(fn_name):
            setattr(self, fn_name, MockFunction(fn_name))
        mock_function = getattr(self, fn_name, None)
        return mock_function.on(*args, **kwargs)

    def assert_expectations(self):
        for attr in dir(self):
            if isinstance(getattr(self, attr), MockFunction):
                mock_function = getattr(self, attr)
                mock_function.assert_expectations()

    def _hasattr(self, name) -> bool:
        attr = getattr(self, name)
        return not isinstance(attr, UnexpectedFunctionInvocation)

    def __getattr__(self, name: str):
        return self.__dict__.get(name, UnexpectedFunctionInvocation(name))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Pearson Service Insight's Logging API Module
This module contains a class with methods to interact with the Logging API.
"""
from __future__ import annotations

import logging
from functools import wraps
from os import environ
from sys import stdout
from typing import Any, Optional


class Logger:
    def __init__(
        self,
        name: str = "psi-logger",
        level: Optional[str] = None,
        logger: Optional[logging.Logger] = None,
    ) -> None:

        formatter = logging.Formatter(
            "%(asctime)s [%(levelname)s] [%(filename)s:%(lineno)d] %(message)s"
        )

        self._level = level or environ.get("LOG_LEVEL", "debug")

        handler = logging.StreamHandler(stdout)
        handler.setFormatter(formatter)

        self._log = logger or logging.getLogger(name)
        self._log.addHandler(handler)
        self._log.propagate = False
        self._set_level(level=self._level)

    def _set_level(self, level: str = "debug") -> None:
        level = level.lower()
        self._level = level
        if level == "info":
            level = logging.INFO
        elif level == "warning":
            level = logging.WARNING
        elif level == "error":
            level = logging.ERROR
        elif level == "critical":
            level = logging.CRITICAL
        elif level == "none":
            level = logging.NOTSET
        else:
            level = logging.DEBUG
        self._log.setLevel(level)

    def log_it(self, func) -> None:
        @wraps(func)
        def __call__(*args, **kwargs):
            self._log.debug(
                f"-- {func.__module__}.{func.__qualname__}(args={args},kwargs={kwargs} --"
            )
            return func(*args, **kwargs)

        return __call__

    def __repr__(self) -> str:
        """Return a string representation of the object"""
        return f"{type(self)}"

    def get_logger(self) -> logging.Logger:
        return self._log

    def __getitem__(self, key) -> Any:
        return self.__getattribute__(key)



import json
from typing import Optional

import pytest
from botocore.exceptions import ClientError

from psi_utils.secrets import MalformedSecretException, SecretsManager
from psi_utils.unittest_mock import MockApi, ANY_NOT_NONE


class TestSecrets_UnitTests:

    @pytest.mark.unit
    def test_encrypt(self):
        # Set expected values
        expected_username = "user"
        expected_password = "password"
        expected_alias = "alias"

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("encrypt", KeyId=expected_alias, Plaintext=expected_password).return_val(
            {"CiphertextBlob": "value".encode("utf-8")}
        )
        mock_session = MockApi()
        mock_session.on("client", "kms").return_val(mock_client)

        # Test Encrypt()
        sm = SecretsManager(session=mock_session)
        username, encrypted = sm.Encrypt(expected_username, expected_password, expected_alias)
        assert username == expected_username
        assert encrypted == b"dmFsdWU="
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_decrypt(self):
        # Set expected values
        expected_secret = "dmFsdWU="

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("decrypt", CiphertextBlob=b"value").return_val({"Plaintext": "value2"})
        mock_session = MockApi()
        mock_session.on("client", "kms").return_val(mock_client)

        # Test Decrypt()
        sm = SecretsManager(session=mock_session)
        plaintext = sm.Decrypt(expected_secret)
        assert plaintext == "value2"
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_pushsecret(self):
        # Set expected values
        expected_key = "secret.id"
        expected_username = "user"
        expected_password = b"password"
        expected_secret_string = json.dumps({"username": expected_username, "password": "password"})
        expected_response = 1

        # Set up mocks
        mock_client = MockApi()
        mock_client.on(
            "put_secret_value",
            SecretId=expected_key,
            ClientRequestToken=ANY_NOT_NONE,
            SecretString=expected_secret_string,
        ).return_val(expected_response)
        mock_session = MockApi()
        mock_session.on("client", "secretsmanager", region_name="us-east-1").return_val(mock_client)

        # Test PushSecret()
        sm = SecretsManager(session=mock_session)
        resp = sm.PushSecret(expected_key, expected_username, expected_password)
        assert resp == expected_response
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_getcredentials(self):
        # Set expected values
        expected_key = "secret.id"
        expected_username = "user"
        expected_password_b64 = "dmFsdWU="
        expected_resp = {
            "SecretString": json.dumps(
                {"username": expected_username, "password": expected_password_b64}
            )
        }

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("get_secret_value", SecretId=expected_key).return_val(expected_resp)
        mock_kms_client = MockApi()
        mock_kms_client.on("decrypt", CiphertextBlob=b"value").return_val({"Plaintext": "value2"})
        mock_session = MockApi()
        mock_session.on(
            "client", service_name="secretsmanager", region_name="us-east-1"
        ).return_val(mock_client)
        mock_session.on("client", "kms").return_val(mock_kms_client)

        # Test GetCredentials()
        sm = SecretsManager(session=mock_session)
        username, password = sm.GetCredentials(expected_key, double_decrypt_password=True)
        assert username == expected_username
        assert password == "value2"
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_getcredentials_no_username(self):
        # Set expected values
        expected_key = "secret.id"
        expected_password_b64 = "dmFsdWU="
        expected_resp = {"SecretString": json.dumps({"password": expected_password_b64})}

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("get_secret_value", SecretId=expected_key).return_val(expected_resp)
        mock_session = MockApi()
        mock_session.on(
            "client", service_name="secretsmanager", region_name="us-east-1"
        ).return_val(mock_client)

        # Test GetCredentials() no username
        sm = SecretsManager(session=mock_session)
        with pytest.raises(MalformedSecretException) as e_info:
            sm.GetCredentials(expected_key)
        assert str(e_info.value) == "secret does not contain a 'username' field"
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_getcredentials_no_password(self):
        # Set expected values
        expected_key = "secret.id"
        expected_resp = {"SecretString": json.dumps({"username": "user"})}

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("get_secret_value", SecretId=expected_key).return_val(expected_resp)
        mock_session = MockApi()
        mock_session.on(
            "client", service_name="secretsmanager", region_name="us-east-1"
        ).return_val(mock_client)

        # Test GetCredentials() no username
        sm = SecretsManager(session=mock_session)
        with pytest.raises(MalformedSecretException) as e_info:
            sm.GetCredentials(expected_key)
        assert str(e_info.value) == "secret does not contain a 'password' field"
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_getcredentials_DecryptionFailureException(self):
        # Set expected values
        expected_key = "secret.id"
        expected_error = self.get_client_error(code="DecryptionFailureException")

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("get_secret_value", SecretId=expected_key).raise_exception(expected_error)
        mock_session = MockApi()
        mock_session.on(
            "client", service_name="secretsmanager", region_name="us-east-1"
        ).return_val(mock_client)

        # Test GetCredentials() DecryptionFailureException
        sm = SecretsManager(session=mock_session)
        with pytest.raises(ClientError) as e_info:
            sm.GetCredentials(expected_key)
        assert e_info.value == expected_error
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_getcredentials_client_error(self):
        # Set expected values
        expected_key = "secret.id"
        expected_error = self.get_client_error(code="UnknownCode")

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("get_secret_value", SecretId=expected_key).raise_exception(expected_error)
        mock_session = MockApi()
        mock_session.on(
            "client", service_name="secretsmanager", region_name="us-east-1"
        ).return_val(mock_client)

        # Test GetCredentials() ClientError
        sm = SecretsManager(session=mock_session)
        with pytest.raises(ClientError) as e_info:
            sm.GetCredentials(expected_key)
        assert e_info.value == expected_error
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_getparameter(self):
        # Set expected values
        expected_param_name = "param.name"
        expected_param = "param.value"

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("get_parameter", Name=expected_param_name, WithDecryption=True).return_val(
            {"Parameter": {"Value": expected_param}}
        )
        mock_session = MockApi()
        mock_session.on("client", service_name="ssm", region_name="us-east-1").return_val(
            mock_client
        )

        # Test GetParameter()
        sm = SecretsManager(session=mock_session)
        assert sm.GetParameter(expected_param_name) == expected_param
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_getparameter_missing_value(self):
        # Set expected values
        expected_param_name = "param.name"

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("get_parameter", Name=expected_param_name, WithDecryption=True).return_val(
            {}
        )
        mock_session = MockApi()
        mock_session.on("client", service_name="ssm", region_name="us-east-1").return_val(
            mock_client
        )

        # Test GetParameter()
        sm = SecretsManager(session=mock_session)
        with pytest.raises(Exception) as e_info:
            sm.GetParameter(expected_param_name)
        assert str(e_info.value) == "Parameter value not found in response"
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_getapikey(self):
        # Set expected values
        expected_username = "user.name@pearson.com"
        expected_result = {"name": expected_username}
        expected_iterator = [{"items": [expected_result]}]

        # Set up mocks
        mock_paginator = MockApi()
        mock_paginator.on(
            "paginate", PaginationConfig={"PageSize": 10}, includeValues=True
        ).return_val(expected_iterator)
        mock_client = MockApi()
        mock_client.on("get_paginator", "get_api_keys").return_val(mock_paginator)
        mock_session = MockApi()
        mock_session.on("client", "apigateway", region_name="us-east-1").return_val(mock_client)

        # Test GetApiKey()
        sm = SecretsManager(session=mock_session)
        assert sm.GetApiKey(expected_username) == expected_result
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_getapikey_no_match(self):
        # Set expected values
        expected_iterator = [{"items": []}]

        # Set up mocks
        mock_paginator = MockApi()
        mock_paginator.on(
            "paginate", PaginationConfig={"PageSize": 10}, includeValues=True
        ).return_val(expected_iterator)
        mock_client = MockApi()
        mock_client.on("get_paginator", "get_api_keys").return_val(mock_paginator)
        mock_session = MockApi()
        mock_session.on("client", "apigateway", region_name="us-east-1").return_val(mock_client)

        # Test GetApiKey() no matching username
        sm = SecretsManager(session=mock_session)
        with pytest.raises(MalformedSecretException) as e_info:
            sm.GetApiKey("user.name@pearson.com")
        assert str(e_info.value).startswith("Unable to find API key")
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_gettoken(self):
        # Set expected values
        expected_key = "token.key"
        expected_token = "token.value"
        expected_secret_json = json.dumps({"token": expected_token})

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("get_secret_value", SecretId=expected_key).return_val(
            {"SecretString": expected_secret_json}
        )
        mock_session = MockApi()
        mock_session.on(
            "client", service_name="secretsmanager", region_name="us-east-1"
        ).return_val(mock_client)

        # Test GetToken()
        sm = SecretsManager(session=mock_session)
        assert sm.GetToken(expected_key) == expected_token
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_gettoken_no_token(self):
        # Set expected values
        expected_key = "token.key"
        expected_secret_json = "{}"

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("get_secret_value", SecretId=expected_key).return_val(
            {"SecretString": expected_secret_json}
        )
        mock_session = MockApi()
        mock_session.on(
            "client", service_name="secretsmanager", region_name="us-east-1"
        ).return_val(mock_client)

        # Test GetToken() no token
        sm = SecretsManager(session=mock_session)
        with pytest.raises(MalformedSecretException) as e_info:
            sm.GetToken(expected_key)
        assert str(e_info.value) == "secret does not contain a 'token' field"
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_gettoken_client_error(self):
        # Set expected values
        expected_key = "token.key"
        expected_error = self.get_client_error(code="UnknownCode")

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("get_secret_value", SecretId=expected_key).raise_exception(expected_error)
        mock_session = MockApi()
        mock_session.on(
            "client", service_name="secretsmanager", region_name="us-east-1"
        ).return_val(mock_client)

        # Test GetToken() ClientError
        sm = SecretsManager(session=mock_session)
        with pytest.raises(Exception) as e_info:
            sm.GetToken(expected_key)
        assert e_info.value == expected_error
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_get_secret(self):
        # Set expected values
        expected_key = "secret.key"
        expected_secret = {"key": "value"}
        expected_secret_json = json.dumps(expected_secret)

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("get_secret_value", SecretId=expected_key).return_val(
            {"SecretString": expected_secret_json}
        )
        mock_session = MockApi()
        mock_session.on(
            "client", service_name="secretsmanager", region_name="us-east-1"
        ).return_val(mock_client)

        # Test GetSecret()
        sm = SecretsManager(session=mock_session)
        assert sm.GetSecret(expected_key) == expected_secret
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_get_secret_no_secret_string(self):
        # Set expected values
        expected_key = "secret.key"

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("get_secret_value", SecretId=expected_key).return_val({})
        mock_session = MockApi()
        mock_session.on(
            "client", service_name="secretsmanager", region_name="us-east-1"
        ).return_val(mock_client)

        # Test GetSecret() no secret string
        sm = SecretsManager(session=mock_session)
        with pytest.raises(MalformedSecretException) as e_info:
            sm.GetSecret(expected_key)
        assert str(e_info.value) == "secret does not contain a SecretString"
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_get_secret_exception(self):
        # Set expected values
        expected_key = "secret.key"
        expected_error = Exception("test")

        # Set up mocks
        mock_client = MockApi()
        mock_client.on("get_secret_value", SecretId=expected_key).raise_exception(expected_error)
        mock_session = MockApi()
        mock_session.on(
            "client", service_name="secretsmanager", region_name="us-east-1"
        ).return_val(mock_client)

        # Test GetSecret() Exception
        sm = SecretsManager(session=mock_session)
        with pytest.raises(Exception) as e_info:
            sm.GetSecret(expected_key)
        assert e_info.value == expected_error
        mock_client.assert_expectations()
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_get_item(self):
        sm = SecretsManager(session=MockApi())
        assert sm["_xray_capture"] is False

    @pytest.mark.unit
    def test_repr(self):
        sm = SecretsManager(session=MockApi())
        assert str(sm) == "<class 'psi_utils.secrets.SecretsManager'>"

    def get_client_error(self, code: str, operation: Optional[str] = None) -> ClientError:
        operation = operation or "test operation"
        return ClientError({"Error": {"Code": code}}, operation)



import json
import pytest
import pytz

from datetime import datetime
from psi_utils.json_utils import JsonDecoder, JsonEncoder


class TestJsonUtils_UnitTests:
    @pytest.mark.unit
    def test_json_encoder(self):
        expected = '{"dt": "2022-01-01 11:12:13.141516-06:00", "date": "2022-01-01"}'
        dt = datetime(2022, 1, 1, 11, 12, 13, 141516)
        dt = pytz.timezone("America/Chicago").localize(dt)
        actual = json.dumps({"dt": dt, "date": dt.date()}, cls=JsonEncoder)
        assert actual == expected

    @pytest.mark.unit
    def test_json_decorder(self):
        expected_dt = datetime(2022, 1, 1, 11, 12, 13, 141516)
        expected_dt_tz = pytz.timezone("America/Chicago").localize(expected_dt)
        expected_date = expected_dt.date()
        expected = {
            "str": "string",
            "date": expected_date,
            "dt": expected_dt,
            "dt_iso": expected_dt,
            "dt_tz": expected_dt_tz,
            "dt_tz_iso": expected_dt_tz,
            "bad_date": "2022-30-40",
            "bad_dt": "2022-30-40 11:12:13.141516",
        }
        actual = json.loads(
            json.dumps(
                {
                    "str": "string",
                    "date": "2022-01-01",
                    "dt": "2022-01-01 11:12:13.141516",
                    "dt_iso": "2022-01-01T11:12:13.141516",
                    "dt_tz": "2022-01-01 11:12:13.141516-06:00",
                    "dt_tz_iso": "2022-01-01T11:12:13.141516-06:00",
                    "bad_date": "2022-30-40",
                    "bad_dt": "2022-30-40 11:12:13.141516",
                }
            ),
            object_hook=JsonDecoder,
        )
        assert actual == expected



import pytest
import pytz

from botocore.exceptions import ClientError
from datetime import datetime
from psi_utils.s3_loader import S3Loader
from psi_utils.unittest_mock import MockApi

TEST_CAPTURE_TIME = datetime.fromisoformat("2021-08-12T13:49:18.763").replace(tzinfo=pytz.utc)


class TestS3Loader_UnitTests:

    @pytest.mark.unit
    def test_load(self):
        # Set expected values
        expected_body = '{"key": "test value"}'
        expected_bucket = "test-bucket"
        expected_key = "scorecards-metrics/test_collector/2021/8/12/1628776158763"

        # Set up mock clients
        mock_client = MockApi()
        mock_client.on(
            "put_object",
            Body=expected_body,
            Bucket=expected_bucket,
            Key=expected_key,
            ACL="bucket-owner-full-control",
        ).return_val(None)
        mock_session = MockApi()
        mock_session.on("client", "s3").return_val(mock_client)

        # Initialize S3Loader and run load()
        loader = S3Loader(bucket=expected_bucket, aws_session=mock_session)
        s3_uri = loader.load(
            collector_name="test_collector",
            data={"key": "test value"},
            capture_time=TEST_CAPTURE_TIME,
        )
        assert s3_uri == f"s3://{expected_bucket}/{expected_key}"

        # Assert expectations
        mock_session.assert_expectations()
        mock_client.assert_expectations()

    @pytest.mark.unit
    def test_load_no_data(self):
        # Set up mock clients
        mock_session = MockApi()
        mock_session.on("client", "s3").return_val(MockApi())

        # Initialize S3Loader and run load()
        loader = S3Loader(bucket="test-bucket", aws_session=mock_session)
        s3_uri = loader.load(
            collector_name="test_collector",
            data={},
            capture_time=TEST_CAPTURE_TIME,
        )
        assert s3_uri == None

        # Assert expectations
        mock_session.assert_expectations()

    @pytest.mark.unit
    def test_put_object(self):
        # Set expected values
        expected_body = '{"key": "test value"}'
        expected_bucket = "test-bucket"
        expected_key = "test-key"
        expected_exception = ClientError({}, "put_object")

        # Set up mock clients
        mock_client = MockApi()
        mock_client.on(
            "put_object",
            Body=expected_body,
            Bucket=expected_bucket,
            Key=expected_key,
            ACL="bucket-owner-full-control",
        ).raise_exception(expected_exception)
        mock_session = MockApi()
        mock_session.on("client", "s3").return_val(mock_client)

        # Assert exception raised when loader.__put_object is called
        loader = S3Loader(bucket=expected_bucket, aws_session=mock_session)
        with pytest.raises(Exception) as e_info:
            loader._S3Loader__put_object(expected_key, expected_body, expected_bucket)
        assert e_info.value == expected_exception

        # Assert expectations
        mock_session.assert_expectations()
        mock_client.assert_expectations()

    @pytest.mark.unit
    def test_dt_to_ms(self):
        # Set up mock clients
        mock_session = MockApi()
        mock_session.on("client", "s3").return_val(MockApi())

        loader = S3Loader(bucket="test-bucket", aws_session=mock_session)
        assert loader._S3Loader__dt_to_ms(TEST_CAPTURE_TIME) == 1628776158763
        assert loader._S3Loader__dt_to_ms(None) == -1



import pytest
import sys

from psi_utils.argparse import ArgumentParser
from psi_utils.unittest_mock import MockApi, ANY_NOT_NONE
from typing import List


class TestArgparse_UnitTests:
    @pytest.mark.unit
    def test_add_argument(self):
        parser = ArgumentParser()
        parser.add_argument("-a", "--alpha", required=True, help="Test arg alpha")
        parser.add_argument("-b", "--beta", default="beta.default")
        subparser = parser.add_subparsers(required=True, dest="subcommand", title="subcommands")
        subparser.add_parser("cmd").add_argument("-g", "--gamma")
        self.set_sys_argv(["-a", "alpha.value", "cmd", "-g", "gamma.value"])
        parser.parse_args()
        parser.print_help()

    @pytest.mark.unit
    def test_parse_args_and_prompt(self):
        parser = ArgumentParser()
        parser.add_argument("-t", "--test", required=True, help="Test argument")
        self.set_sys_argv(["-t", "test.value"])
        with pytest.raises(OSError) as e_info:
            parser.parse_args_and_prompt()
        assert str(e_info.value).startswith("pytest: reading from stdin while output is captured")
        self.reset_sys_argv()

    @pytest.mark.unit
    def test_print_help(self):
        parser = ArgumentParser(default_help_format=True)
        parser.print_help()

        parser = ArgumentParser()
        parser.print_help()

        mock_file = MockApi()
        mock_file.on("write", ANY_NOT_NONE).return_val(None)
        parser.print_help(file=mock_file)
        mock_file.assert_expectations()

    def set_sys_argv(self, args: List[str]):
        self.__sys_argv = sys.argv
        sys.argv = [self.__sys_argv[0]] + args

    def reset_sys_argv(self):
        if hasattr(self, "__sys_argv"):
            sys.argv = self.__sys_argv



import argparse
import sys

INDENT = "    "
BOLD = "\033[1m"
FORMAT_END = "\033[0m"

TEMPLATE = f"""
{BOLD}usage{FORMAT_END}: <<usage>>

{BOLD}DESCRIPTION{FORMAT_END}

{INDENT}<<description>>

{BOLD}REQUIRED PARAMETERS{FORMAT_END}
<<required>>
{BOLD}OPTIONAL PARAMETERS{FORMAT_END}
<<optional>>"""


def format_parameters(params: list) -> str:
    msg = "No parameters."
    if params:
        msg = ""
        for i in range(len(params)):
            msg += "\n"
            param = params[i]
            options_string = BOLD + ", ".join(param.option_strings) + FORMAT_END
            is_flag = type(param) in [
                argparse._StoreTrueAction,
                argparse._StoreFalseAction,
                argparse._HelpAction,
            ]
            options_string += "" if is_flag else " [arg]"
            help_string = param.help or "No help message."
            msg += f"{BOLD}{options_string}{FORMAT_END}\n\n"
            msg += f"{help_string}\n"
            if param.default and "--help" not in param.option_strings:
                msg += f"{BOLD}Default:{FORMAT_END} {param.default}\n"

    # Indent all lines
    msg = "\n".join([f"{INDENT}{s}" for s in msg.split("\n")])
    return msg


def format_help(parser) -> str:
    msg = TEMPLATE

    # Format description
    description_str = parser.description or "No description."

    # Format usage
    usage_str = parser.usage
    if not usage_str:
        usage_str = f"python {sys.argv[0]}"
        if len(parser._action_groups) > 0 and len(parser._action_groups[0]._group_actions) > 0:
            usage_str += " [commands]"
        if len(parser._action_groups) > 1 and len(parser._action_groups[1]._group_actions) > 0:
            usage_str += " [parameters]"

    # TODO: handle commands

    # Format parameters
    required_params, optional_params = [], []
    if len(parser._action_groups) > 1:
        required_params = [e for e in parser._action_groups[1]._group_actions if e.required]
        optional_params = [e for e in parser._action_groups[1]._group_actions if not e.required]
    required_params_str = format_parameters(required_params)
    optional_params_str = format_parameters(optional_params)

    msg = msg.replace("<<description>>", description_str)
    msg = msg.replace("<<usage>>", usage_str)
    msg = msg.replace("<<required>>", required_params_str)
    msg = msg.replace("<<optional>>", optional_params_str)
    return msg



import logging
import pytest

from psi_utils.logger import Logger


class TestLogger_UnitTests:

    @pytest.mark.unit
    def test_logger(self):
        # Test init, _set_level(), and get_logger()
        logger = Logger()
        assert logger.get_logger().level == logging.DEBUG
        logger = Logger(level="info")
        assert logger.get_logger().level == logging.INFO
        logger = Logger(level="warning")
        assert logger.get_logger().level == logging.WARNING
        logger = Logger(level="error")
        assert logger.get_logger().level == logging.ERROR
        logger = Logger(level="critical")
        assert logger.get_logger().level == logging.CRITICAL
        logger = Logger(level="none")
        assert logger.get_logger().level == logging.NOTSET
        logger._set_level("info")

        # Test log_it
        @logger.log_it
        def test_fn():
            pass

        test_fn()

        # Test __repr__
        assert str(logger) == "<class 'psi_utils.logger.Logger'>"

        # Test __getitem__()
        assert logger["_level"] == "info"



import json
import pytz

from datetime import datetime


class JsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if type(obj) not in [int, float, complex, dict, tuple, list, bool] and obj is not None:
            return str(obj)
        return json.JSONEncoder.default(self, obj)


def JsonDecoder(d: dict):
    """Json decoder that parses date and datetime strings into date and datetime objects

    Accepts 2 date formats:
     - Default python3.8 datetime format, e.g.: 2022-01-01 11:12:13.141516
     - ISO format, e.g.: 2022-01-01T11:12:13.141516
    Also handles timezone suffixes like '-06:00'
    """
    date_str_len = len("YYYY-mm-dd")
    dt_str_len = len("YYYY-mm-dd HH:MM:SS.ffffff")
    dt_with_tz_str_len = len("YYYY-mm-dd HH:MM:SS.ffffff+00:00")
    for k, v in d.items():
        if isinstance(v, str):
            if len(v) == date_str_len:
                try:
                    dt = datetime.strptime(v, "%Y-%m-%d").replace(tzinfo=pytz.utc).date()
                    d[k] = dt
                except Exception:
                    pass
            elif len(v) in [dt_str_len, dt_with_tz_str_len]:
                try:
                    temp = v.replace(" ", "T")
                    dt = datetime.fromisoformat(temp)
                    d[k] = dt
                except Exception:
                    pass
    return d



import json
from datetime import datetime
from typing import Optional, Union

import boto3
from botocore.exceptions import ClientError

from psi_utils.logger import Logger
from psi_utils.json_utils import JsonEncoder


class S3Loader:
    def __init__(
        self,
        bucket: Optional[str] = None,
        path: Optional[str] = None,
        aws_session: Optional[boto3.Session] = None,
        s3_client=None,
        logger=None,
    ):
        self.__bucket = bucket or "psi-scorecard-metrics-us-east-1-490126044637"
        self.__path = path or "scorecards-metrics"
        self.__aws_session = aws_session or boto3.Session()
        self.__s3_client = s3_client or self.__aws_session.client("s3")
        self.__logger = logger or Logger("psi-s3-loader").get_logger()

    def load(self, collector_name: str, data: Union[dict, list], capture_time: datetime) -> str:
        """Load data into S3. Returns the S3 uri if successful."""
        if data:
            assert collector_name, "Cannot load S3 data without a collector name"
            assert capture_time, "Cannot load S3 data without a capture time"
            assert isinstance(data, list) or isinstance(
                data, dict
            ), f"Failed to load S3 data. Expected data to be a dict or list, got: {type(data)}"
            capture_date = capture_time.date()
            capture_time_ms = self.__dt_to_ms(capture_time)
            key = f"{self.__path}/{collector_name}/{capture_date.year}/{capture_date.month}/{capture_date.day}/{capture_time_ms}"
            return self.__put_object(
                key=key, body=json.dumps(data, cls=JsonEncoder), bucket=self.__bucket
            )
        return None

    def __put_object(self, key: str, body: str, bucket: str) -> str:
        """Put object into an S3 bucket. Returns the S3 uri if successful"""
        try:
            self.__s3_client.put_object(
                Body=body, Bucket=bucket, Key=key, ACL="bucket-owner-full-control"
            )
            return f"s3://{bucket}/{key}"
        except ClientError as e:
            self.__logger.error(f"Failed to put S3 object s3://{bucket}/{key} with exception: {e}")
            raise e

    def __dt_to_ms(self, dt: datetime) -> int:
        if dt:
            return int(round(dt.timestamp() * 1000))
        return -1
